KEY_EVENT = dict()
KEY_EVENT['KEYCODE_SOFT_LEFT'] = 1
KEY_EVENT['KEYCODE_SOFT_RIGHT'] = 2
KEY_EVENT['KEYCODE_HOME'] = 3
KEY_EVENT['KEYCODE_BACK'] = 4
KEY_EVENT['KEYCODE_CALL'] = 5
KEY_EVENT['KEYCODE_ENDCALL'] = 6
KEY_EVENT['KEYCODE_0'] = 7
KEY_EVENT['KEYCODE_1'] = 8
KEY_EVENT['KEYCODE_2'] = 9
KEY_EVENT['KEYCODE_3'] = 10
KEY_EVENT['KEYCODE_4'] = 11
KEY_EVENT['KEYCODE_5'] = 12
KEY_EVENT['KEYCODE_6'] = 13
KEY_EVENT['KEYCODE_7'] = 14
KEY_EVENT['KEYCODE_8'] = 15
KEY_EVENT['KEYCODE_9'] = 16
KEY_EVENT['KEYCODE_STAR'] = 17
KEY_EVENT['KEYCODE_POUND'] = 18
KEY_EVENT['KEYCODE_DPAD_UP'] = 19
KEY_EVENT['KEYCODE_DPAD_DOWN'] = 20
KEY_EVENT['KEYCODE_DPAD_LEFT'] = 21
KEY_EVENT['KEYCODE_DPAD_RIGHT'] = 22
KEY_EVENT['KEYCODE_DPAD_CENTER'] = 23
KEY_EVENT['KEYCODE_VOLUME_UP'] = 24
KEY_EVENT['KEYCODE_VOLUME_DOWN'] = 25
KEY_EVENT['KEYCODE_POWER'] = 26
KEY_EVENT['KEYCODE_CAMERA'] = 27
KEY_EVENT['KEYCODE_CLEAR'] = 28
KEY_EVENT['KEYCODE_A'] = 29
KEY_EVENT['KEYCODE_B'] = 30
KEY_EVENT['KEYCODE_C'] = 31
KEY_EVENT['KEYCODE_D'] = 32
KEY_EVENT['KEYCODE_E'] = 33
KEY_EVENT['KEYCODE_F'] = 34
KEY_EVENT['KEYCODE_G'] = 35
KEY_EVENT['KEYCODE_H'] = 36
KEY_EVENT['KEYCODE_I'] = 37
KEY_EVENT['KEYCODE_J'] = 38
KEY_EVENT['KEYCODE_K'] = 39
KEY_EVENT['KEYCODE_L'] = 40
KEY_EVENT['KEYCODE_M'] = 41
KEY_EVENT['KEYCODE_N'] = 42
KEY_EVENT['KEYCODE_O'] = 43
KEY_EVENT['KEYCODE_P'] = 44
KEY_EVENT['KEYCODE_Q'] = 45
KEY_EVENT['KEYCODE_R'] = 46
KEY_EVENT['KEYCODE_S'] = 47
KEY_EVENT['KEYCODE_T'] = 48
KEY_EVENT['KEYCODE_U'] = 49
KEY_EVENT['KEYCODE_V'] = 50
KEY_EVENT['KEYCODE_W'] = 51
KEY_EVENT['KEYCODE_X'] = 52
KEY_EVENT['KEYCODE_Y'] = 53
KEY_EVENT['KEYCODE_Z'] = 54
KEY_EVENT['KEYCODE_COMMA'] = 55
KEY_EVENT['KEYCODE_PERIOD'] = 56
KEY_EVENT['KEYCODE_ALT_LEFT'] = 57
KEY_EVENT['KEYCODE_ALT_RIGHT'] = 58
KEY_EVENT['KEYCODE_SHIFT_LEFT'] = 59
KEY_EVENT['KEYCODE_SHIFT_RIGHT'] = 60
KEY_EVENT['KEYCODE_TAB'] = 61
KEY_EVENT['KEYCODE_SPACE'] = 62
KEY_EVENT['KEYCODE_SYM'] = 63
KEY_EVENT['KEYCODE_EXPLORER'] = 64
KEY_EVENT['KEYCODE_ENVELOPE'] = 65
KEY_EVENT['KEYCODE_ENTER'] = 66
KEY_EVENT['KEYCODE_DEL'] = 67
KEY_EVENT['KEYCODE_GRAVE'] = 68
KEY_EVENT['KEYCODE_MINUS'] = 69
KEY_EVENT['KEYCODE_EQUALS'] = 70
KEY_EVENT['KEYCODE_LEFT_BRACKET'] = 71
KEY_EVENT['KEYCODE_RIGHT_BRACKET'] = 72
KEY_EVENT['KEYCODE_BACKSLASH'] = 73
KEY_EVENT['KEYCODE_SEMICOLON'] = 74
KEY_EVENT['KEYCODE_APOSTROPHE'] = 75
KEY_EVENT['KEYCODE_SLASH'] = 76
KEY_EVENT['KEYCODE_AT'] = 77
KEY_EVENT['KEYCODE_NUM'] = 78
KEY_EVENT['KEYCODE_HEADSETHOOK'] = 79
KEY_EVENT['KEYCODE_FOCUS'] = 80
KEY_EVENT['KEYCODE_PLUS'] = 81
KEY_EVENT['KEYCODE_MENU'] = 82
KEY_EVENT['KEYCODE_NOTIFICATION'] = 83
KEY_EVENT['KEYCODE_SEARCH'] = 84
KEY_EVENT['KEYCODE_MEDIA_PLAY_PAUSE'] = 85
KEY_EVENT['KEYCODE_MEDIA_STOP'] = 86
KEY_EVENT['KEYCODE_MEDIA_NEXT'] = 87
KEY_EVENT['KEYCODE_MEDIA_PREVIOUS'] = 88
KEY_EVENT['KEYCODE_MEDIA_REWIND'] = 89
KEY_EVENT['KEYCODE_MEDIA_FAST_FORWARD'] = 90
KEY_EVENT['KEYCODE_MUTE'] = 91
KEY_EVENT['KEYCODE_PAGE_UP'] = 92
KEY_EVENT['KEYCODE_PAGE_DOWN'] = 93
KEY_EVENT['KEYCODE_PICTSYMBOLS'] = 94
KEY_EVENT['KEYCODE_SWITCH_CHARSET'] = 95
KEY_EVENT['KEYCODE_BUTTON_A'] = 96
KEY_EVENT['KEYCODE_BUTTON_B'] = 97
KEY_EVENT['KEYCODE_BUTTON_C'] = 98
KEY_EVENT['KEYCODE_BUTTON_X'] = 99
KEY_EVENT['KEYCODE_BUTTON_Y'] = 100
KEY_EVENT['KEYCODE_BUTTON_Z'] = 101
KEY_EVENT['KEYCODE_BUTTON_L1'] = 102
KEY_EVENT['KEYCODE_BUTTON_R1'] = 103
KEY_EVENT['KEYCODE_BUTTON_L2'] = 104
KEY_EVENT['KEYCODE_BUTTON_R2'] = 105
KEY_EVENT['KEYCODE_BUTTON_THUMBL'] = 106
KEY_EVENT['KEYCODE_BUTTON_THUMBR'] = 107
KEY_EVENT['KEYCODE_BUTTON_START'] = 108
KEY_EVENT['KEYCODE_BUTTON_SELECT'] = 109
KEY_EVENT['KEYCODE_BUTTON_MODE'] = 110
KEY_EVENT['KEYCODE_ESCAPE'] = 111
KEY_EVENT['KEYCODE_FORWARD_DEL'] = 112
KEY_EVENT['KEYCODE_CTRL_LEFT'] = 113
KEY_EVENT['KEYCODE_CTRL_RIGHT'] = 114
KEY_EVENT['KEYCODE_CAPS_LOCK'] = 115
KEY_EVENT['KEYCODE_SCROLL_LOCK'] = 116
KEY_EVENT['KEYCODE_META_LEFT'] = 117
KEY_EVENT['KEYCODE_META_RIGHT'] = 118
KEY_EVENT['KEYCODE_FUNCTION'] = 119
KEY_EVENT['KEYCODE_SYSRQ'] = 120
KEY_EVENT['KEYCODE_BREAK'] = 121
KEY_EVENT['KEYCODE_MOVE_HOME'] = 122
KEY_EVENT['KEYCODE_MOVE_END'] = 123
KEY_EVENT['KEYCODE_INSERT'] = 124
KEY_EVENT['KEYCODE_FORWARD'] = 125
KEY_EVENT['KEYCODE_MEDIA_PLAY'] = 126
KEY_EVENT['KEYCODE_MEDIA_PAUSE'] = 127
KEY_EVENT['KEYCODE_MEDIA_CLOSE'] = 128
KEY_EVENT['KEYCODE_MEDIA_EJECT'] = 129
KEY_EVENT['KEYCODE_MEDIA_RECORD'] = 130
KEY_EVENT['KEYCODE_F1'] = 131
KEY_EVENT['KEYCODE_F2'] = 132
KEY_EVENT['KEYCODE_F3'] = 133
KEY_EVENT['KEYCODE_F4'] = 134
KEY_EVENT['KEYCODE_F5'] = 135
KEY_EVENT['KEYCODE_F6'] = 136
KEY_EVENT['KEYCODE_F7'] = 137
KEY_EVENT['KEYCODE_F8'] = 138
KEY_EVENT['KEYCODE_F9'] = 139
KEY_EVENT['KEYCODE_F10'] = 140
KEY_EVENT['KEYCODE_F11'] = 141
KEY_EVENT['KEYCODE_F12'] = 142
KEY_EVENT['KEYCODE_NUM_LOCK'] = 143
KEY_EVENT['KEYCODE_NUMPAD_0'] = 144
KEY_EVENT['KEYCODE_NUMPAD_1'] = 145
KEY_EVENT['KEYCODE_NUMPAD_2'] = 146
KEY_EVENT['KEYCODE_NUMPAD_3'] = 147
KEY_EVENT['KEYCODE_NUMPAD_4'] = 148
KEY_EVENT['KEYCODE_NUMPAD_5'] = 149
KEY_EVENT['KEYCODE_NUMPAD_6'] = 150
KEY_EVENT['KEYCODE_NUMPAD_7'] = 151
KEY_EVENT['KEYCODE_NUMPAD_8'] = 152
KEY_EVENT['KEYCODE_NUMPAD_9'] = 153
KEY_EVENT['KEYCODE_NUMPAD_DIVIDE'] = 154
KEY_EVENT['KEYCODE_NUMPAD_MULTIPLY'] = 155
KEY_EVENT['KEYCODE_NUMPAD_SUBTRACT'] = 156
KEY_EVENT['KEYCODE_NUMPAD_ADD'] = 157
KEY_EVENT['KEYCODE_NUMPAD_DOT'] = 158
KEY_EVENT['KEYCODE_NUMPAD_COMMA'] = 159
KEY_EVENT['KEYCODE_NUMPAD_ENTER'] = 160
KEY_EVENT['KEYCODE_NUMPAD_EQUALS'] = 161
KEY_EVENT['KEYCODE_NUMPAD_LEFT_PAREN'] = 162
KEY_EVENT['KEYCODE_NUMPAD_RIGHT_PAREN'] = 163
KEY_EVENT['KEYCODE_VOLUME_MUTE'] = 164
KEY_EVENT['KEYCODE_INFO'] = 165
KEY_EVENT['KEYCODE_CHANNEL_UP'] = 166
KEY_EVENT['KEYCODE_CHANNEL_DOWN'] = 167
KEY_EVENT['KEYCODE_ZOOM_IN'] = 168
KEY_EVENT['KEYCODE_ZOOM_OUT'] = 169
KEY_EVENT['KEYCODE_TV'] = 170
KEY_EVENT['KEYCODE_WINDOW'] = 171
KEY_EVENT['KEYCODE_GUIDE'] = 172
KEY_EVENT['KEYCODE_DVR'] = 173
KEY_EVENT['KEYCODE_BOOKMARK'] = 174
KEY_EVENT['KEYCODE_CAPTIONS'] = 175
KEY_EVENT['KEYCODE_SETTINGS'] = 176
KEY_EVENT['KEYCODE_TV_POWER'] = 177
KEY_EVENT['KEYCODE_TV_INPUT'] = 178
KEY_EVENT['KEYCODE_STB_POWER'] = 179
KEY_EVENT['KEYCODE_STB_INPUT'] = 180
KEY_EVENT['KEYCODE_AVR_POWER'] = 181
KEY_EVENT['KEYCODE_AVR_INPUT'] = 182
KEY_EVENT['KEYCODE_PROG_RED'] = 183
KEY_EVENT['KEYCODE_PROG_GREEN'] = 184
KEY_EVENT['KEYCODE_PROG_YELLOW'] = 185
KEY_EVENT['KEYCODE_PROG_BLUE'] = 186
KEY_EVENT['KEYCODE_APP_SWITCH'] = 187
KEY_EVENT['KEYCODE_BUTTON_1'] = 188
KEY_EVENT['KEYCODE_BUTTON_2'] = 189
KEY_EVENT['KEYCODE_BUTTON_3'] = 190
KEY_EVENT['KEYCODE_BUTTON_4'] = 191
KEY_EVENT['KEYCODE_BUTTON_5'] = 192
KEY_EVENT['KEYCODE_BUTTON_6'] = 193
KEY_EVENT['KEYCODE_BUTTON_7'] = 194
KEY_EVENT['KEYCODE_BUTTON_8'] = 195
KEY_EVENT['KEYCODE_BUTTON_9'] = 196
KEY_EVENT['KEYCODE_BUTTON_10'] = 197
KEY_EVENT['KEYCODE_BUTTON_11'] = 198
KEY_EVENT['KEYCODE_BUTTON_12'] = 199
KEY_EVENT['KEYCODE_BUTTON_13'] = 200
KEY_EVENT['KEYCODE_BUTTON_14'] = 201
KEY_EVENT['KEYCODE_BUTTON_15'] = 202
KEY_EVENT['KEYCODE_BUTTON_16'] = 203
KEY_EVENT['KEYCODE_LANGUAGE_SWITCH'] = 204
KEY_EVENT['KEYCODE_MANNER_MODE'] = 205
KEY_EVENT['KEYCODE_3D_MODE'] = 206
KEY_EVENT['KEYCODE_CONTACTS'] = 207
KEY_EVENT['KEYCODE_CALENDAR'] = 208
KEY_EVENT['KEYCODE_MUSIC'] = 209
KEY_EVENT['KEYCODE_CALCULATOR'] = 210
KEY_EVENT['KEYCODE_ZENKAKU_HANKAKU'] = 211
KEY_EVENT['KEYCODE_EISU'] = 212
KEY_EVENT['KEYCODE_MUHENKAN'] = 213
KEY_EVENT['KEYCODE_HENKAN'] = 214
KEY_EVENT['KEYCODE_KATAKANA_HIRAGANA'] = 215
KEY_EVENT['KEYCODE_YEN'] = 216
KEY_EVENT['KEYCODE_RO'] = 217
KEY_EVENT['KEYCODE_KANA'] = 218
KEY_EVENT['KEYCODE_ASSIST'] = 219
KEY_EVENT['KEYCODE_BRIGHTNESS_DOWN'] = 220
KEY_EVENT['KEYCODE_BRIGHTNESS_UP'] = 221
KEY_EVENT['KEYCODE_MEDIA_AUDIO_TRACK'] = 222
KEY_EVENT['KEYCODE_SLEEP'] = 223
KEY_EVENT['KEYCODE_WAKEUP'] = 224
KEY_EVENT['KEYCODE_PAIRING'] = 225
KEY_EVENT['KEYCODE_MEDIA_TOP_MENU'] = 226
KEY_EVENT['KEYCODE_11'] = 227
KEY_EVENT['KEYCODE_12'] = 228
KEY_EVENT['KEYCODE_LAST_CHANNEL'] = 229
KEY_EVENT['KEYCODE_TV_DATA_SERVICE'] = 230
KEY_EVENT['KEYCODE_VOICE_ASSIST'] = 231
KEY_EVENT['KEYCODE_TV_RADIO_SERVICE'] = 232
KEY_EVENT['KEYCODE_TV_TELETEXT'] = 233
KEY_EVENT['KEYCODE_TV_NUMBER_ENTRY'] = 234
KEY_EVENT['KEYCODE_TV_TERRESTRIAL_ANALOG'] = 235
KEY_EVENT['KEYCODE_TV_TERRESTRIAL_DIGITAL'] = 236
KEY_EVENT['KEYCODE_TV_SATELLITE'] = 237
KEY_EVENT['KEYCODE_TV_SATELLITE_BS'] = 238
KEY_EVENT['KEYCODE_TV_SATELLITE_CS'] = 239
KEY_EVENT['KEYCODE_TV_SATELLITE_SERVICE'] = 240
KEY_EVENT['KEYCODE_TV_NETWORK'] = 241
KEY_EVENT['KEYCODE_TV_ANTENNA_CABLE'] = 242
KEY_EVENT['KEYCODE_TV_INPUT_HDMI_1'] = 243
KEY_EVENT['KEYCODE_TV_INPUT_HDMI_2'] = 244
KEY_EVENT['KEYCODE_TV_INPUT_HDMI_3'] = 245
KEY_EVENT['KEYCODE_TV_INPUT_HDMI_4'] = 246
KEY_EVENT['KEYCODE_TV_INPUT_COMPOSITE_1'] = 247
KEY_EVENT['KEYCODE_TV_INPUT_COMPOSITE_2'] = 248
KEY_EVENT['KEYCODE_TV_INPUT_COMPONENT_1'] = 249
KEY_EVENT['KEYCODE_TV_INPUT_COMPONENT_2'] = 250
KEY_EVENT['KEYCODE_TV_INPUT_VGA_1'] = 251
KEY_EVENT['KEYCODE_TV_AUDIO_DESCRIPTION'] = 252
KEY_EVENT['KEYCODE_TV_AUDIO_DESCRIPTION_MIX_UP'] = 253
KEY_EVENT['KEYCODE_TV_AUDIO_DESCRIPTION_MIX_DOWN'] = 254
KEY_EVENT['KEYCODE_TV_ZOOM_MODE'] = 255
KEY_EVENT['KEYCODE_TV_CONTENTS_MENU'] = 256
KEY_EVENT['KEYCODE_TV_MEDIA_CONTEXT_MENU'] = 257
KEY_EVENT['KEYCODE_TV_TIMER_PROGRAMMING'] = 258
KEY_EVENT['KEYCODE_HELP'] = 259
KEY_EVENT['KEYCODE_NAVIGATE_PREVIOUS'] = 260
KEY_EVENT['KEYCODE_NAVIGATE_NEXT'] = 261
KEY_EVENT['KEYCODE_NAVIGATE_IN'] = 262
KEY_EVENT['KEYCODE_NAVIGATE_OUT'] = 263
KEY_EVENT['KEYCODE_STEM_PRIMARY'] = 264
KEY_EVENT['KEYCODE_STEM_1'] = 265
KEY_EVENT['KEYCODE_STEM_2'] = 266
KEY_EVENT['KEYCODE_STEM_3'] = 267
KEY_EVENT['KEYCODE_DPAD_UP_LEFT'] = 268
KEY_EVENT['KEYCODE_DPAD_DOWN_LEFT'] = 269
KEY_EVENT['KEYCODE_DPAD_UP_RIGHT'] = 270
KEY_EVENT['KEYCODE_DPAD_DOWN_RIGHT'] = 271
KEY_EVENT['KEYCODE_MEDIA_SKIP_FORWARD'] = 272
KEY_EVENT['KEYCODE_MEDIA_SKIP_BACKWARD'] = 273
KEY_EVENT['KEYCODE_MEDIA_STEP_FORWARD'] = 274
KEY_EVENT['KEYCODE_MEDIA_STEP_BACKWARD'] = 275
KEY_EVENT['KEYCODE_SOFT_SLEEP'] = 276
KEY_EVENT['KEYCODE_CUT'] = 277
KEY_EVENT['KEYCODE_COPY'] = 278
KEY_EVENT['KEYCODE_PASTE'] = 279
KEY_EVENT['KEYCODE_SYSTEM_NAVIGATION_UP'] = 280
KEY_EVENT['KEYCODE_SYSTEM_NAVIGATION_DOWN'] = 281
KEY_EVENT['KEYCODE_SYSTEM_NAVIGATION_LEFT'] = 282
KEY_EVENT['KEYCODE_SYSTEM_NAVIGATION_RIGHT'] = 283
KEY_EVENT['KEYCODE_ALL_APPS'] = 284
KEY_EVENT['KEYCODE_REFRESH'] = 285
KEY_EVENT['KEYCODE_THUMBS_UP'] = 286
KEY_EVENT['KEYCODE_THUMBS_DOWN'] = 287
KEY_EVENT['KEYCODE_PROFILE_SWITCH'] = 288
KEY_EVENT['KEYCODE_VIDEO_APP_1'] = 289
KEY_EVENT['KEYCODE_VIDEO_APP_2'] = 290
KEY_EVENT['KEYCODE_VIDEO_APP_3'] = 291
KEY_EVENT['KEYCODE_VIDEO_APP_4'] = 292
KEY_EVENT['KEYCODE_VIDEO_APP_5'] = 293
KEY_EVENT['KEYCODE_VIDEO_APP_6'] = 294
KEY_EVENT['KEYCODE_VIDEO_APP_7'] = 295
KEY_EVENT['KEYCODE_VIDEO_APP_8'] = 296
KEY_EVENT['KEYCODE_FEATURED_APP_1'] = 297
KEY_EVENT['KEYCODE_FEATURED_APP_2'] = 298
KEY_EVENT['KEYCODE_FEATURED_APP_3'] = 299
KEY_EVENT['KEYCODE_FEATURED_APP_4'] = 300
KEY_EVENT['KEYCODE_DEMO_APP_1'] = 301
KEY_EVENT['KEYCODE_DEMO_APP_2'] = 302
KEY_EVENT['KEYCODE_DEMO_APP_3'] = 303
KEY_EVENT['KEYCODE_DEMO_APP_4'] = 304
